@@ -1,4 +1,4 @@
-/*	$Id: bind.c,v 1.33 2001/08/10 13:19:55 sandro Exp $	*/
+/*	$Id: bind.c,v 1.2 2003/04/24 15:11:59 rrt Exp $	*/
 
 /*
  * Copyright (c) 1997-2001 Sandro Sigala.  All rights reserved.
@@ -221,12 +221,12 @@ int do_completion(char *s, int *compl)
 	if (!*compl) {
 		c = cur_tp->xgetkey(GETKEY_DELAYED, 500);
 		if (c == KBD_NOKEY) {
-			minibuf_write(buf);
+			minibuf_write("%s", buf);
 			c = cur_tp->getkey();
 			*compl = 1;
 		}
 	} else {
-		minibuf_write(buf);
+		minibuf_write("%s", buf);
 		c = cur_tp->getkey();
 	}
 	minibuf_clear();
