@@ -1,4 +1,4 @@
-/*	$Id: buffer.c,v 1.28 2001/07/31 12:10:55 sandro Exp $	*/
+/*	$Id: buffer.c,v 1.2 2003/04/24 15:11:59 rrt Exp $	*/
 
 /*
  * Copyright (c) 1997-2001 Sandro Sigala.  All rights reserved.
@@ -146,9 +146,8 @@ bufferp create_buffer(const char *name)
 	bp = new_buffer();
 	set_buffer_name(bp, name);
 
-	if (head_bp == cur_bp)
-		head_bp = bp;
-	bp->next = cur_bp;
+	bp->next = head_bp;
+        head_bp = bp;
 
 	return bp;
 }
