@@ -41,9 +41,10 @@ extern void	set_temporary_buffer(bufferp bp);
 /* file.c ----------------------------------------------------------------- */
 extern int	exist_file(char *filename);
 extern int	is_regular_file(char *filename);
-extern int	expand_path(char *path, char *cwdir, char *dir, char *fname);
-extern char *	compact_path(char *buf, char *path);
-extern char *	get_current_dir(char *buf);
+extern int	expand_path(char *path, char *cwdir, pathbuffer_t *dir,
+			    pathbuffer_t *fname);
+extern pathbuffer_t *	compact_path(pathbuffer_t *buf, char *path);
+extern pathbuffer_t *	get_current_dir(pathbuffer_t *buf);
 extern void	open_file(char *path, int lineno);
 extern void	read_from_disk(char *filename);
 extern int	find_file(char *filename);
@@ -96,8 +97,11 @@ extern linep	resize_line(windowp wp, linep lp, int maxsize);
 extern void	free_line(linep lp);
 extern void	line_replace_text(linep *lp, int offset, int orgsize, char *newtext);
 extern int	insert_char(int c);
+extern int      insert_char_in_insert_mode(int c);
+extern int      intercalate_char(int c);
 extern int	insert_tab(void);
 extern int	insert_newline(void);
+extern int      intercalate_newline(void);
 extern void	insert_string(char *s);
 extern void	insert_nstring(char *s, size_t size);
 extern int	self_insert_command(int c);
