@@ -1,4 +1,4 @@
-/*	$Id: fontlock.c,v 1.15 2001/07/31 12:10:55 sandro Exp $	*/
+/*	$Id: fontlock.c,v 1.2 2003/04/24 15:11:59 rrt Exp $	*/
 
 /*
  * Copyright (c) 1997-2001 Sandro Sigala.  All rights reserved.
@@ -111,13 +111,13 @@ static void cpp_set_anchors(linep lp, int *lastanchor)
 			 * cannot be longer than a line.
 			 */
 			*ap++ = ANCHOR_NULL;
-			do {
+			while (sp < lp->text + lp->size - 1 && *sp != '\'') {
 				*ap++ = ANCHOR_NULL;
 				if (*++sp == '\\' && sp < lp->text + lp->size - 1) {
 					*ap++ = ANCHOR_NULL;
 					++sp;
 				}
-			} while (sp < lp->text + lp->size - 1 && *sp != '\'');
+			}
 		} else if (*sp == '"') {
 			/*
 			 * Found a string beginning sequence.  Put an anchor to
