@@ -1,4 +1,4 @@
-/*	$Id: search.c,v 1.14 2001/07/31 12:10:55 sandro Exp $	*/
+/*	$Id: search.c,v 1.2 2003/04/24 15:11:59 rrt Exp $	*/
 
 /*
  * Copyright (c) 1997-2001 Sandro Sigala.  All rights reserved.
@@ -223,7 +223,7 @@ static int isearch(int dir)
 			 last ? "" : "Failing ",
 			 (dir == ISEARCH_FORWARD) ? "" : " backward",
 			 astr_cstr(pattern));
-		minibuf_write(astr_cstr(buf));
+		minibuf_write("%s", astr_cstr(buf));
 		c = cur_tp->getkey();
 		if (c == KBD_CANCEL) {
 			cur_wp->pointp = startp;
