@@ -1,4 +1,4 @@
-/*	$Id: zile.h,v 1.47 2001/07/06 16:33:02 sandro Exp $	*/
+/*	$Id: zile.h,v 1.4 2003/04/24 15:47:40 rrt Exp $	*/
 
 /*
  * Copyright (c) 1997-2001 Sandro Sigala.  All rights reserved.
@@ -28,6 +28,7 @@
 #define ZILE_H
 
 #include <alist.h>
+#include "pathbuffer.h"
 
 #undef TRUE
 #define TRUE				1
@@ -103,6 +104,8 @@ struct line {
 #define UNDO_START_SEQUENCE		7
 /* End a multi operation sequence. */
 #define UNDO_END_SEQUENCE		8
+/* Insert a char without moving the current pointer */
+#define UNDO_INTERCALATE_CHAR           9
 
 struct undo {
 	/* Next undo delta in list. */
@@ -259,7 +262,7 @@ struct history {
 
 	/* This flag is set when this is a filename history. */
 	int	fl_dir;
-	char	*path;
+	pathbuffer_t	*path;
 
 	/* This flag is set when the space character is allowed. */
 	int	fl_space;
